% FIGURE_2a_2b creates Figure(2a) and Figure(2b) 

% CASE: General Model: Economic and Ecological Uncertainty with non-zero
% decay rate

% The numerical method is based on solving the variational inequality
% associated with the problem.

clear all
optset('newton','showiters',1)
optset('newton','maxit',200)

disp(' ')
disp('Solving Optimal Timing Problems in Environmental Economics')
disp('FIGURE (2a) and FIGURE (2b)')
disp(' ')

% Define problem parameters
clear all
alpha  = 0.01;          % drift rate on social cost
r      = 0.04;          % discount rate
delta  = 0.0;           % annual pollution decay rate 
sigma1 = 0.1;           % annual social cost volatility
sigma2 = 1;             % annual pollution stock standard deviation (million tons)
beta   = 1;             % emissions to stock conversion factor
E0     = 0.3;           % initial emissions level (million tons)
K      = 4000;          % conversion cost (million $)
k      = K/E0;

%% Set up the approximation space and define the number of approximation nodes
Tmin=0; Tmax=2.1;           % theta limits
Mmin=0; Mmax=101;           % M limits
Tnodes=71; Mnodes=51;       % number of approximation nodes in each dimension

%% Choose upwind or centered finite difference derivatives
%Upwinding method and piecewise linear are highly recomended for convergence issues.
upwind=1;                   % 1 for upwind, 2 for centered finite difference
family=1;                   % 1 for piecewise linear, 2 for cubic spline

if (family==1),             %define approximaion family 
    af='lin'; 
else 
    af='spli'; 
end

sigma=[0.1,0.20,0.40];
for iter=1:length(sigma), % loop over solutions of models with different parametrization (# of combinations is 3)
    sigma1=sigma(iter)
 
    fspace=fundefn(af,[Tnodes Mnodes],[Tmin Mmin],[Tmax Mmax]);

    snodes=funnode(fspace);     % define the nodes in state space (dimension=2)
    s=gridmake(snodes);         % create the grid with the nodal points
                                % T=S(:,1), M=S(:,2)
                                
    %% construct the basis functions in each dimension including 1st & 2nd derivatives
    bases=funbasx(fspace,snodes,[0;1;2]);
    % The solution to the problem is the solution to the function, 0=min(M_1*c+q_1,M_2*c+q_2)

    
    %% compute the drift and the diffusion parameters at each nodal point
    n=size(s,1);
    mu = [alpha*s(:,1), beta*E0-delta.*s(:,2)];
    Sigma = zeros(n,2,2);
    Sigma(:,1,1) = sigma1*s(:,1);
    Sigma(:,2,2) = sigma2;

    % construct the matrices for the complementarity approach
    if (upwind==1) && (family==1)
        h=zeros(1,2);
        Sigma=sigma2Sigma(Sigma,2,0);
        for i=1:2, 
            h(i)=snodes{i}(2)-snodes{i}(1); 
        end
        M_1=linpdeU(r*ones(n,1),mu,Sigma,[Tnodes Mnodes],h);
        M_2=linpdeU(ones(n,1),[zeros(n,1), zeros(n,1)],zeros(n,2,2),[Tnodes Mnodes],h);
    else
        M_1 = ctbasemake(r,bases,0)-ctbasemake(mu,bases,1)-ctbasemake(Sigma,bases,2);
        M_2 = ctbasemake(ones(n,1),bases,0);
    end

    q_1 = s(:,1).*s(:,2).^2;
    q_2 = s(:,1).*(s(:,2).^2+sigma2.^2./(r-alpha))./(r+2*delta-alpha)+K;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Impose boundary conditions, V(0,M)=0
    snodesb=cell(1,2);
    snodesb{1,1}=snodes{1,1}(1,1);snodesb{1,2}=snodes{1,2};
    sb=gridmake(snodesb);
    [temp,indb]=nearest(s,sb,1);
    basesb=funbasx(fspace,snodesb,[0;1;2]);
    M_1b=ctbasemake(1,basesb,0);
    q_1b=0;
    M_1(indb,:)=M_1b;
    q_1(indb)=q_1b;
    M_2(indb,:)=M_1b;
    q_2(indb)=q_1b;
    %
    if iter==1,
        %% solve for initial approximation
        warning ('off','MATLAB:divideByZero')
        Mstar=r*k*(r+2*delta-alpha)/(2*beta)./s(:,1); 
        x00=double(s(:,2)<Mstar);   
        warning ('on','MATLAB:divideByZero')

        %%
        % Pack model structure for the deterministic case
        clear model
        model.func='Pindyckmff';
        model.C='variable';
        model.sigmadiag=1;
        model.values=[0 NaN 0];

        % SETUP for good starting values using the deterministic case 
        % nonstochastic (sigma1=sigma2=0) case, obtain starting values
        model.params={alpha,r,delta,0,0,beta,E0,k};

        optset('optstopsolve','alg','f');
        
        % solve for deterministic case
        [cv0,snodes,x0,w0,err]=optstopsolve(model,fspace,snodes,[],x00);
        v=funeval(cv0,fspace,snodes);

        A=diagmult(1-x0,M_1)+diagmult(x0,M_2);
        b=(1-x0).*q_1+x0.*q_2;
        cv=-(A\b);
    end;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% solve problem using EVLCP solver such as SMOOTHING NEWTON
    maxit=500;
    showiters=1;
    [cnew,w,err,it] = snewton([M_1; M_2],[q_1, q_2],cv,[],[],[],[],[],maxit,[],showiters);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %% Get optimal stopping rule
    [res,xindex]=min([M_1*cnew+q_1,M_2*cnew+q_2],[],2);
    
    x=xindex-1; %If x=0 >> Continuation region
                %If x=1 >> Invest with infinite rate and hit the free boundary 
    
    res=reshape(res,Tnodes,Mnodes);

    %% compute the approximate value function    
    Tnodes=Tnodes+1;Mnodes=Mnodes+1; %this creates better figures.

    ss=nodeunif([Tnodes Mnodes],[Tmin Mmin],[Tmax Mmax]);
    V=funeval(cnew, fspace,ss);
    V=reshape(V,Tnodes,Mnodes);             % Value Function

    x1=linspace(Tmin,Tmax,Tnodes);
    x2=linspace(Mmin,Mmax,Mnodes);
    [T, M]=ndgrid(x1,x2);

    %% Pindyck's Closed Form Solutions
    Mx2=linspace(0,Mmax)';
    [thetastar, gam1]=pindyk02analytic(Mx2, r, alpha, K, sigma1, sigma2, beta, E0);

    % Known value function in "Adopt" Region
    V_A=-T.*(M.^2+sigma2.^2./(r-alpha))./(r+2*delta-alpha);
    
    
    %% Create the Plots %%
    if sigma1==0.1 | sigma1==0.2 |sigma1==0.4,
        if iter == 1,
            figure('Color',[1 1 1]);
        else
            figure(1),
        end
        hold on 
        plot(Mx2,thetastar,'Color',[0 0 0],'LineWidth',2)
            xlabel('Current Pollutant Stock, M (million tons)')
            ylabel('Threshold, \theta^*(M)')
            title('Stopping Boundaries')
            ylim([0,2])
            xlim([0,100])
        
        if iter == 1,
            figure('Color',[1 1 1]);
        else
            figure(2),
        end
        hold on 
        contour(M,T,V-V_A+K,[0,0],'LineWidth',2,'LineColor',[0 0 0])
            xlabel('Current Pollutant Stock, M (million tons)')
            ylabel('Threshold, \theta^*(M)')
            title('Stopping Boundaries')
            ylim([0,2])
            xlim([0,100])
    end;
    Tnodes=Tnodes-1;Mnodes=Mnodes-1;
    
    %Use the approximation coefficients in the next iteration to improve convergence
    cv=cnew;
    
end;


%% Create textarrows for the figures
% For Figure-a
annotation1 = annotation(1,'textarrow',[0.3983 0.1912],[0.8547 0.7046],...
  'String',{'\sigma_1=0.1'});
annotation2 = annotation(1,'textarrow',[0.4322 0.2322],[0.7048 0.5667],...
  'String',{'\sigma_1=0.2'});
annotation3 = annotation(1,'textarrow',[0.4947 0.2089],[0.3786 0.1524],...
  'String',{'\sigma_1=0.4'});
% For Figure-b
annotation1 = annotation(2,'textarrow',[0.3839 0.1769],[0.8667 0.7166],...
  'String',{'\sigma_1=0.1'});
annotation2 = annotation(2,'textarrow',[0.4072 0.2072],[0.7358 0.5977],...
  'String',{'\sigma_1=0.2'});
annotation3 = annotation(2,'textarrow',[0.4375 0.3107],[0.619 0.5286],...
  'String',{'\sigma_1=0.4'});
