#include "mex.h"
#include <math.h>
/* 
% fishermin Fisher function phiminus = y+z-sqrt(y^2+z^2)
% USAGE:
%   [phi,d1,d2]=fishermin(y,z);
% INPUTS
%   y,z : mxn matrices
% OUPUTS
%   phi : mxn matrix of function values (evaluated element by element)
%   d1, d2 : mxn matrices of derivatives with respect to y and z,
%               respectively
%
% Note: The derivatives at (0,0) is not well defined but are set here to 1
%          (the average of the one-sides derivatives)
*/


/* 
   Fisher function phiminus = y+z-sqrt(y^2+z^2)
*/
double fishermin(double y, double z){
double e, e2, se;
if (fabs(y)<=fabs(z))
  if (y==0){ 
    if (z>=0) return(0);
    else      return(2*z);
  }
  else{
    e=y/z;
    e2=e*e;
    se=sqrt(1+e2);
    if (z<0)
      return(z*(1+se+e));
    else
      return(z*(e-((1-se)*(1-se)+e2)/(2*se)));
  }
else
  if (z==0){
    if (y>=0) return(0);
    else      return(2*y);
  }
  else{
    e=z/y;
    e2=e*e;
    se=sqrt(1+e2);
    if (y<0)
      return(y*(1+se+e));
    else
      return(y*(e-((1-se)*(1-se)+e2)/(2*se)));
  }
}

/* 
   Fisher function phiminus = y+z-sqrt(y^2+z^2)
   with derivatives
*/
double fishermind(double y, double z, double *d1, double *d2){
double e, e2, se;
if (fabs(y)<=fabs(z))
  if (y==0)
    if      (z>0) {*d1 = 1; *d2=0; return(0);}
    else if (z<0) {*d1 = 1; *d2=2; return(2*z);}
    else          {*d1 = 1; *d2=1; return(0);}
  else{
    e=y/z;
    e2=e*e;
    se=sqrt(1+e2);
    if (z<0){
      *d2=1+1/se; *d1=1+e/se;
      return(z*(1+se+e));
    }
    else{
      *d2=1-1/se; *d1=1-e/se;
      return(z*(e-((1-se)*(1-se)+e2)/(2*se)));
    }
  }
else
  if (z==0)
    if      (y>0) {*d2 = 1; *d1=0; return(0);}
    else if (y<0) {*d2 = 1; *d1=2; return(2*y);}
    else          {*d2 = 1; *d1=1; return(0);}
  else{
    e=z/y;
    e2=e*e;
    se=sqrt(1+e2);
    if (y<0){
      *d1=1+1/se; *d2=1+e/se;
      return(y*(1+se+e));
    }
    else{
      *d1=1-1/se; *d2=1-e/se;
      return(y*(e-((1-se)*(1-se)+e2)/(2*se)));
    } 
  }
}

void mexFunction(
   int nlhs, mxArray *plhs[],
   int nrhs, const mxArray *prhs[])
{
  int m,n,i;
  double *y, *z, *phi, *d1, *d2;
   
  if (nrhs<2)               mexErrMsgTxt("Two arguments must be passed.");
  if (nlhs>3)               mexErrMsgTxt("Either one or three outputs are created");
  if (nlhs==2)              mexErrMsgTxt("Either one or three outputs are created"); 
  if (!mxIsDouble(prhs[0])) mexErrMsgTxt("Inputs must be double");
  if (!mxIsDouble(prhs[1])) mexErrMsgTxt("Inputs must be double");
  if (mxIsSparse(prhs[0]))  mexErrMsgTxt("Inputs must be full (not sparse)");
  if (mxIsSparse(prhs[1]))  mexErrMsgTxt("Inputs must be full (not sparse)");
  if (mxIsComplex(prhs[0])) mexErrMsgTxt("Inputs must be real");
  if (mxIsComplex(prhs[1])) mexErrMsgTxt("Inputs must be real");
  m=mxGetM(prhs[0]);
  n=mxGetN(prhs[0]);
  if (mxGetM(prhs[1])!=m)   mexErrMsgTxt("Inputs must be the same size");
  if (mxGetN(prhs[1])!=n)   mexErrMsgTxt("Inputs must be the same size");
  y=mxGetPr(prhs[0]);
  z=mxGetPr(prhs[1]);
  
  plhs[0]=mxCreateDoubleMatrix(m,n,mxREAL);
  phi=mxGetPr(plhs[0]);
  if (nlhs>1){ 
    plhs[1]=mxCreateDoubleMatrix(m,n,mxREAL);
    plhs[2]=mxCreateDoubleMatrix(m,n,mxREAL);
    d1=mxGetPr(plhs[1]);
    d2=mxGetPr(plhs[2]);
    m*=n;
    for (i=0; i<m; i++)
      *phi++=fishermind(*y++,*z++,d1++,d2++);
  }
  else{
    m*=n;
    for (i=0; i<m; i++)
      *phi++=fishermin(*y++,*z++);
  }  
}
