% GLCPSOLVE Solves a generalized linear complementarity problem (GLCP)
% Solves 0=min(M1*x+q1,M2*x+q2)
% Uses the Fisher function reformation: 
%   0 = x+y+sqrt(x^2+y^2)
% where x=M1*x+q1 and y=M2*x+q2
% The ordinary LCP has M2=I and q2=0.
% USAGE
%   [x,w,err]=glcpsolve(x,M1,q1,M2,q2);
% INPUTS
%   x   : n-vector: initial guess of the solution
%   M1  : nxn matrix (full or sparse)
%   q1  : nx1 vector
%   M2  : nxn matrix (full or sparse)
%   q2  : nx1 vector
% OUTPUTS
%   x   : n-zector: solution to the LCP\
%   w   : nx2 matrix containing [M1*x+q1 M2*x+q2]
%   err : error code
%            0 indicates sucess
%            1 indicates that the maximum number of interations was exceeded
%
% Settable options (using OPTSET);
%   maxit     : maximum number of major iterations [100]
%   tol       : convergence tolerance (norm(f,inf)<tol) [sqrt(eps)]
%   maxsteps  : maximum number of minor interations (backsteps) [10]
%   showiters : displays progress infomration at each major iteration [0]
%   itcheck   : finite termination check after itcheck major iterations [20]

function [x,w,err]=glcpsolve(x,M1,q1,M2,q2)

maxit     = optget('glcpsolve','maxit',100);
tol       = optget('glcpsolve','tol',sqrt(eps));
maxsteps  = optget('glcpsolve','maxsteps',25);
showiters = optget('glcpsolve','showiters',1);
itcheck   = optget('glcpsolve','itcheck',20);
err=0;

for it=1:maxit
   w1=M1*x+q1; w2=M2*x+q2; 
   [fval,fjac] = ff(w1,w2,M1,M2);
   fnorm = norm(fval,inf);
   %fnorm = fval'*fval;
   if fnorm<tol, break, end
   dx = -(fjac\fval);
   fnormold = inf;  
   for backstep=1:maxsteps 
     xdx=x+dx;
     w1=M1*xdx+q1; w2=M2*xdx+q2;
     fvalnew = ff(w1,w2);
     fnormnew = norm(fvalnew,inf);
     %fnormnew = fvalnew'*fvalnew;
     if fnormnew<fnorm, break, end
     if fnormold<fnormnew, fnormnew=fnormold; dx=2*dx; break, end
     fnormold = fnormnew;
     dx = dx/2;          
   end
   x = x+dx;
   if showiters, fprintf('%4i %4i %6.2e\n',[it backstep fnormnew]); end
   % finite termination procedure
   if it>itcheck | fnormnew<10*tol 
     w1=M1*x+q1; w2=M2*x+q2;% w3=M3*x+q3; 
     %w2=min([w2, w3],[],2);
     i1=double(w1<w2);
     M=diagmult(i1,M1)+diagmult(1-i1,M2);
     q=q1.*(i1)+q2.*(1-i1); 
     warning off
     xx=-(M\q);
     warning on
     w1=M1*xx+q1; w2=M2*xx+q2;
     i2=double(w1<w2);
     if all(i1==i2) & ~any(isnan(xx))
       x=xx;
       w=[w1 w2];
       return
     end
   end
end
if it>=maxit
  if nargout<3 
    warning('Failure to converge in glcpsolve'); 
  else
    err=1;
  end
end
w=[M1*x+q1 M2*x+q2];
return


% computes the Fisher function reformulation of the GLCP
function [f,df]=ff(w1,w2,M1,M2);
if nargout>1
  [f,d1,d2]=fishermin(w1,w2);
  df=diagmult(d1,M1)+diagmult(d2,M2);
else 
  f=fishermin(w1,w2);
end

