% LINPDEU Creates upwind finite difference matrix operators for solving linear PDEs
% USAGE
%   M=linpdeU(rho,mu,Sigma,n,h);
% INPUTS
%  rho   : Nx1 vector of coefficients for the order 0 terms
%  mu    : 1xd cell arrays composed on Nx1 vectors for the order 1 terms
%             Alternatively mu can be Nxd
%  Sigma : dxd cell array composed of Nx1 vectors for the order 2 terms
%             (only the lower triangular portion is used) 
%             Alternatively sigma can be Nxdxd
%  n     : d-vector of number of nodal values for each variable
%  h     : d-vector of step sizes for each variable
% OUTPUT
%  B   : NxN matrix which B*v approximates the PDE operator
%            rho(s)V(s)-V'(s)mu(s)-0.5 vec(V''(S))'vec(sigma(s)*sigma(s)')
%        where v(ij)=V(s(ij)) and s(i,j)=s0(j)+(i-1)*h(j), i=1,...,n(j)
%        (ij) is a single index and (i,j) is a double index
%           see SUB2IND and IND2SUB for discussion)

function B=linpdeU(rho,mu,Sigma,n,h)
  d=length(n);  % problem dimension
  N=prod(n);    % problem size
  
  % all dimensions are the same so less storage is required
  if all(n==n(1) & h==h(1)) 
    ni=n(1);
    hi=h(1);
    Phi=repmat({speye(ni)},1,d);
    % forward difference operator
    Df=repmat({sparse([1:ni-1 1:ni-1 ni ni ni],[2:ni 1:ni-1  ni-2 ni-1 ni],...
                 [ones(1,ni-1) -ones(1,ni-1) 1 -3 2]/hi,ni,ni)},1,d);
    % backward difference operator
    Db=repmat({sparse([1 1 1 2:ni 2:ni],[1 2 3 2:ni 1:ni-1],...
                      [-2 3 -1 ones(1,ni-1) -ones(1,ni-1)]/hi,ni,ni)},1,d);
    % central difference operator
    Dc=repmat({sparse([1 1 1 2:ni-1 2:ni-1 ni ni ni],[1 2 3 3:ni 1:ni-2 ni-2 ni-1 ni],...
                      [-3 4 -1 ones(1,ni-2) -ones(1,ni-2) 1 -4 3]/hi,ni,ni)},1,d);                 
  % not all dimensions are the same - store all dimensions separately
  else
    Phi=cell(1,d);
    Df=cell(1,d);
    Db=cell(1,d);
    Dc=cell(1,d);
    for i=1:d
      ni=n(i);
      hi=h(i);
      di=d-i+1;
      Phi{di}=speye(ni);
      % forward difference operator
      Df{di}=sparse([1:ni-1 1:ni-1 ni ni ni],[2:ni 1:ni-1  ni-2 ni-1 ni],...
                    [ones(1,ni-1) -ones(1,ni-1) 1 -3 2]/hi,ni,ni);
      % backward difference operator
      Db{di}=sparse([1 1 1 2:ni 2:ni],[1 2 3 2:ni 1:ni-1],...
                    [-2 3 -1 ones(1,ni-1) -ones(1,ni-1)]/hi,ni,ni);
      % central difference operator
      Dc{di}=sparse([1 1 1 2:ni-1 2:ni-1 ni ni ni],[1 2 3 3:ni 1:ni-2 ni-2 ni-1 ni],...
                    [-3 4 -1 ones(1,ni-2) -ones(1,ni-2) 1 -4 3]/hi,ni,ni);
    end
  end
  
  % Order 0 terms
  if ~isempty(rho), B=sparse(1:N,1:N,rho,N,N);
  else              B=sparse(N,N);
  end
  
  % Order 1 terms
  Di=Phi; 
  for i=1:d
    if isa(mu,'cell'), mui=mu{i};
    else,              mui=mu(:,i);
    end
    if ~isempty(mui)
      di=d-i+1;
      % positive drift rates
      ii=find(mui>0);
      if ~isempty(ii)
        Di{di}=Df{di}; Bi=ckron(Di);
        temp=zeros(size(mui));
        temp(ii)=mui(ii);
        B=B-diagmult(temp,Bi);
      end
      % negative drift rates
      ii=find(mui<0);
      if ~isempty(ii)
        Di{di}=Db{di}; Bi=ckron(Di);
        temp=zeros(size(mui));
        temp(ii)=mui(ii);
        B=B-diagmult(temp,Bi);
      end
      Di{di}=Phi{di};
    end
  end

  % Order 2 terms
  for j=1:d
    if isa(Sigma,'cell')
      Sigmaij=Sigma{j,j};
    else
      Sigmaij=Sigma(:,j,j);
    end
    dj=d-j+1;
    % get jth second derivative term
    if ~isempty(Sigmaij)
      Di{dj}=Db{dj}*Df{dj}; 
      Di{dj}(1,[1 2 3 4])  =[2 -5 4 -1]/h(j)^2;  % correction for left endpoint
      Di{dj}(end,end-3:end)=[-1 4 -5 2]/h(j)^2;  % correction for right endpoint
      Bi=ckron(Di);
      B=B-diagmult(Sigmaij/2,Bi);
    end
    for i=j+1:d
      if isa(Sigma,'cell')
        Sigmaij=Sigma{i,j};
      else
        Sigmaij=Sigma(:,i,j);
      end
      % get ijth cross derivative term
      if ~isempty(Sigmaij)
        di=d-i+1;
        % negative cross terms
        ii=find(Sigmaij<0);
        if ~isempty(ii)
          Bicalc=1;
          Di{dj}=Df{dj}; Di{di}=Db{di}; Bi=ckron(Di);
          Di{dj}=Db{dj}; Di{di}=Df{di}; Bi=Bi+ckron(Di);
          temp=zeros(size(Sigmaij));
          temp(ii)=Sigmaij(ii)/2;
          B=B-diagmult(temp,Bi);
        else
          Bicalc=0;
        end
        % positive cross terms
        ii=find(Sigmaij>0);
        if ~isempty(ii)
          if ~Bicalc 
            Di{dj}=Df{dj}; Di{di}=Df{di}; Bi=ckron(Di);
            Di{dj}=Db{dj}; Di{di}=Db{di}; Bi=Bi+ckron(Di);
          else  % save a step using alternative formula
            Di{dj}=Dc{dj}; Di{di}=Dc{di}; Bi=ckron(Di)-Bi;
          end
          temp=zeros(size(Sigmaij));
          temp(ii)=Sigmaij(ii)/2;
          B=B-diagmult(temp,Bi);
        end
        Di{di}=Phi{di};
      end
    end
    Di{dj}=Phi{dj};
  end
    
  
 