% OPTSTOPSOLVE Solves continuous time optimal switching models
% USAGE
%   [cv,snodes,x]=optstopsolve(model,fspace,snodes,cv,x);
% INPUTS
%   model    : a structure variable (defined below)
%   fspace   : an approximation function structure 
%   [snodes] : nodal state values (n rows)
%   [cv]     : initial guess of solution coefficients (n by 1)
%   [x]      : initial guess of decision rule (0-wait, 1=stop)
% OUTPUTS
%   cv       : solution coefficients (n by 1)
%   snodes   : nodal state values (n rows or cell array)
%   x        : optimal control values (n by 1)
%
% The model structure should contain the following fields:
%   func     : problem definition file (see below)  
%   params   : additional parameters to pass to prob. def. file
%   C        : stopping costs (m by m) or string 'variable'
%   values   : kxd+1 matrix of known values or string 'variable'
%                 use NaN to indicate all values of a variable
%                 Example:
%                    model.values=[0 NaN 0;NaN 0 0];
%                 indicates in a d=2 model that V(0,S2)=V(S1,0)=1
%
% The problem definition file should have the format
%   function out=dscfile(flag,s,R,additional parameters)
%      switch flag
%        case 'f'
%          Return a matrix f representing the reward function
%        case 'mu'
%          Return a matrix mu representing the drift function
%              of the state transition equation
%        case 'sigma'
%          Return a matrix sigma representing the diffusion function
%              of the state transition equation
%        case 'rho'
%          Return a vector representing the (state contingent)
%              discount rates (or a scalar if discount rate is constant)
%      end
% For problems with d continuous states the sizes of these matrices
% should be as follows (with n=# of rows in s):
%            f     : n x 1
%            mu    : n x d
%            sigma : n x d x d (or n x d^2)
%            rho   : n x 1 (or 1 x 1)
% Optional flag when stopping costs are non-constant:
%            C     : n x m
% Settable options (using OPTSET)
%   'alg' : ['snewton'], 'fisher', 'lemke'
%   'upwind' : [1] - uses upwind finite differences
%               0  - uses centered finite differences
%              This only applies if fspace is of type 'lin'
function [cv,snodes,x,w,err]=optstopsolve(model,fspace,snodes,cv,x)
tic;

alg    = optget('optstopsolve','alg','snewton');
upwind = optget('optstopsolve','upwind',1);

  if nargin<3 | isempty(snodes), snodes=funnode(fspace); end
  if nargin<4, cv=[]; end
  if nargin<5, x =[]; end
  d=fspace.d;  % dimension of the state 
  
  % Get basis matrices and nodal points
  S=gridmake(snodes);
  n=prod(fspace.n);

% Unpack model variables
  func=model.func;                                                       % the name of the model function file
  if isfield(model,'params'); params=model.params; else params={}; end   % additional parameters to pass to func
  
  if isfield(model,'sigmadiag'); sigmadiag=model.sigmadiag; else sigmadiag=0; end

  % Check for preset value function
  if isfield(model,'values'), values = model.values; else values = []; end
  
% Compute M and q
  rho=feval(func,'rho',S,[],params{:});
  mu=feval(func,'mu',S,1,params{:});
  sigma=feval(func,'sigma',S,1,params{:});
  if strcmp(fspace.bastype{1},'lin')         % use upwinding bases
    Sigma=sigma2Sigma(sigma,d,sigmadiag);
    h=zeros(1,d);
    for i=1:d, h(i)=snodes{i}(2)-snodes{i}(1); end
    if upwind
      M1=linpdeU(rho,mu,Sigma,fspace.n,h);
    else
      M1=linpdeC(rho,mu,Sigma,fspace.n,h);
    end
    M2=speye(n);
    clear Sigma
  else
    bases=funbasx(fspace,snodes,[0;1;2]);
    if isempty(sigma)
      M1=ctbasemake(rho,bases,0)-ctbasemake(mu,bases,1);
    else
      M1=ctbasemake(rho,bases,0);
      M1=M1-ctbasemake(mu,bases,1);
      M1=M1-ctbasemake(sigma,bases,2);
    end
    M2=ckron(bases.vals(1,d:-1:1));
    clear bases
  end
  q(:,1)=-feval(func,'f',S,1,params{:});
  q(:,2)=feval(func,'C',S,1,params{:});

  clear rho mu sigma
  
  % Alter any rows at which value function is known
  if ~isempty(values)
    if size(values,2)~=d+1 
      error('model.values has the wrong number of columns')
    end
    Svals=expandlist(values,snodes);
    Vvals=-Svals(:,end);
    Svals(:,end)=[];
    [temp,ind]=nearest(S,Svals,1);
    M1temp=M1(ind,:);
    qtemp=q(ind,:);
    q(ind,1)=Vvals;
    q(ind,2)=Vvals;
    M1(ind,:)=funbas(fspace,Svals);
    clear Svals Vvals temp
  end
  
  clear S
  
  
  % Get starting values
  if isempty(cv) 
    if isempty(x)
      cv=-(M1\q(:,1));
    else
      A=diagmult(1-x,M1)+diagmult(x,M2);
      b=(1-x).*q(:,1)+x.*q(:,2);
      cv=-(A\b);
    end
  end
  
  tt=toc;
  fprintf('Time elapsed for setup:  %10.2f\n',tt)
  tic;

  % call the EVLCP solver - either lemkeg, fisher or snewton
  switch upper(alg(1))
  case 'L'
    [cv,w,err,iter]=lemkeg([M1;M2],q); 
  case 'F'
    [cv,w,err]=glcpsolve(cv,M1,q(:,1),M2,q(:,2));
  otherwise
    [cv,w,err,iter]=snewton([M1;M2],q,cv); 
  end
  % Get optimal stopping rule
  if ~isempty(values)
    w(ind,1)=M1temp*cv+qtemp(:,1);
    w(ind,2)=M2(ind,:)*cv+qtemp(:,2);
  end
  [res,xindex]=min(w,[],2);
  x=xindex-1;
  
  tt=toc;
  fprintf('Time elapsed for solver: %10.2f\n',tt)
  return
  

% Converts sigma to Sigma=sigma*sigma'
function Sigma=sigma2Sigma(sigma,d,sigmadiag)
% sigma is not diagonal
if nargin<3 || sigmadiag==0
  if isa(sigma,'cell')
    Sigma=cell(d,d);
    for j=1:d
      for i=1:j
        Sigma{i,j}=sigma{i,1}.*sigma{j,1};
        for k=2:d
          Sigma{i,j}=Sigma{i,j}+sigma{i,k}.*sigma{j,k};
        end
      end
    end
  else
    n=size(sigma);
    if  length(n)==2, sigma=reshape(sigma,n(1),d,d); end
    Sigma=cell(d,d);
    for j=1:d
      for i=1:j
        Sigma{i,j}=sigma(:,i,1).*sigma(:,j,1);
        for k=2:d
          Sigma{i,j}=Sigma{i,j}+sigma(:,i,k).*sigma(:,j,k);
        end
      end
    end
  end
else
  Sigma=cell(d,d);
  if isa(sigma,'cell')
    for i=1:d, Sigma{i,i}=sigma{i,i}.^2; end
  else
    ns=size(sigma);
    if length(ns)==2, sigma=reshape(sigma,ns(1),d,d); end
    for i=1:d, Sigma{i,i}=sigma(:,i,i).^2; end
  end
end
return
    

